"""
This module contains all the utilities needed to interface NPK with NMRNoteBook (copyright 2005 NMRtec Montpellier France)

"""


from NPK import *

#---------------------------------------------------------------------------
# Transformation en dictionnaire d'une table de ptes JAVA
#---------------------------------------------------------------------------
def todict(props):
    """conversion d'une table de proprietes  en dictionnary
    on a besoin de ca pour gerer le processing"""
    dico = {}
    en = props.propertyNames()
    while(en.hasMoreElements()):
        key = en.nextElement()
        dkey = key.lower()
        val = props.getProperty(key)
        try:
            fval = Float.parseFloat(val)
            dico[dkey] = fval
            #print( "*** ", repr(dkey), " -> ", repr(fval))
        except:
            if ( val == "false" ):
                dico[dkey] = 0
            elif ( val == "true" ):
                dico[dkey] = 1
            else:
                dico[dkey] = val
    
    return dico

#---------------------------------------------------------------------------
# do_process_1d
#---------------------------------------------------------------------------
def do_process_1d(auditfile, _p_in, _f_in, _f_out, inputfilename, outputfilename):
    """
    1D processing for NNB
    """
    import sys
    import os.path
    
    # ouverture du fichier audit
    audit = auditinitial(auditfile,"1D processing")
    
    audittrail(audit, "phase", "Loading parameters")

    audittrail(audit, "text", "open Input Spectrum" )
    f_in = todict( _f_in )
    
    audittrail(audit, "text", "open processing parameters ")
    p_in = todict( _p_in)
    p_in["select_state"]=1
    if ( p_in["modulus"] ) :
        p_in["f1_state"]="real"
    else:
        p_in["f1_state"]="complex"
    
    audittrail(audit, "text", "open Output Spectrum ")
    f_out = todict( _f_out)

    audittrail(audit, "phase", "FID preparation phase")
    pre_ft_1d(audit, inputfilename, "memory", p_in, f_in, f_out)

    audittrail(audit, "phase", "Spectral analysis phase")
    ft_1d(audit, "memory", "memory", p_in, f_in, f_out)

    audittrail(audit, "phase", "Post processing phase")
    post_ft_1d(audit, "memory", outputfilename, p_in, f_in, f_out)

    #bidouille pour la phase
    if (p_in["autophase"]):
        spec_in = _f_in["javadata"]
        spec_out = _f_out["javadata"]
        spec_in.getNNB_NMR_Spectrum().getAxisF(1).setPhaseOrder0(f_out["autophase_0"])
        spec_in.getNNB_NMR_Spectrum().getAxisF(1).setPhaseOrder1(f_out["autophase_1"])
        spec_out.getNNB_NMR_Spectrum().getAxisF(1).setPhaseOrder0(f_out["autophase_0"])
        spec_out.getNNB_NMR_Spectrum().getAxisF(1).setPhaseOrder1(f_out["autophase_1"])
        
    if (p_in["phase"]):
        spec_in = _f_in["javadata"]
        spec_out = _f_out["javadata"]
        spec_in.getNNB_NMR_Spectrum().getAxisF(1).setPhaseOrder0(p_in["phase_0"])
        spec_in.getNNB_NMR_Spectrum().getAxisF(1).setPhaseOrder1(p_in["phase_1"])
        spec_out.getNNB_NMR_Spectrum().getAxisF(1).setPhaseOrder0(p_in["phase_0"])
        spec_out.getNNB_NMR_Spectrum().getAxisF(1).setPhaseOrder1(p_in["phase_1"])
    	
    audittrail(audit, "phase", "End of processing")


    # analysis phase
    audittrail(audit, "phase", "1D Analysis")
    audittrail(audit, "phase", "Peak Picking and Integration")
    processing = _p_in["javadata"]
    if (processing.isMakePeakPicking()):
    	audittrail(audit, "phase", "Peak Picking phase")
        tmpar = os.path.split( outputfilename )
        peak_out = os.path.join( tmpar[0], "tmp", "peak.gtb")
    	pp_1d( audit, "memory", peak_out, p_in, f_in, f_out)
    	
    	if (processing.isMakeIntegration()):
    		audittrail(audit, "phase", "Integration phase")
                tmpar = os.path.split( outputfilename )
                integrat_out = os.path.join( tmpar[0], "tmp", "integrate.gtb")
    		integ_1d(audit, "memory", integrat_out, p_in, f_in, f_out)


    audittrail(audit, "phase", "End of analysis")
    audittrail(audit, "close")
    # fermeture du fichier d'audit
    audit.close()
    

    
#---------------------------------------------------------------------------
# do_process_2d 
#---------------------------------------------------------------------------
def do_process_2d(auditfile, _p_in, _f_in, _f_out, inputfilename, outputfilename):
    """
Do a complete 2D processing, interface for the NMRNoteBook program:

    2D FT - Peak-Picking - integration

arguments :
auditfile       the name of the audit file to create
filein          the name of the FID file
fileout         the name of the spectrum file
processing_in   the name of the java property list containing processing parameters 
spec_in         the name of the java property list containing fid details
spec_out        the name of the java property list containing final spectrum details
location        directory where to put projections.
    """

    # ouverture du fichier d'audit
    audit = auditinitial(auditfile,"2D processing")
    
    audittrail(audit, "phase", "Loading Parameters")
    
    audittrail(audit, "text", "open FID ")
    f_in = todict(_f_in)
    
    audittrail(audit, "text", "open processing parameters ")
    p_in = todict(_p_in)
    p_in["select_state"] = 1
    if ( p_in["modulus"] ) :
        p_in["f1_state"]="real"
        p_in["f2_state"] = "real"
    else:
        p_in["f1_state"]="complex"
        p_in["f2_state"] = "complex"
    
    audittrail(audit, "text", "open Spectra ")
    f_out = todict(_f_out)

    audittrail(audit, "phase", "FID analysis phase")
    pre_2d(audit, inputfilename, "memory", p_in, f_in, f_out)

    audittrail(audit, "phase", "F2_axis - Pre FT phase")
    pre_ft_2df2(audit, "memory", "memory", p_in, f_in, f_out)

    audittrail(audit, "phase", "F2 axis - FT phase")
    ft_2df2(audit, "memory", "memory", p_in, f_in, f_out )

    audittrail(audit, "phase", "F2 axis - Post FT phase")
    post_ft_2df2(audit, "memory", "memory", p_in, f_in, f_out )

    audittrail(audit, "phase", "F1_axis - Pre FT phase")
    pre_ft_2df1(audit, "memory", "memory", p_in, f_in, f_out )

    audittrail(audit, "phase", "F1 axis - FT phase")
    ft_2df1(audit, "memory", "memory", p_in, f_in, f_out )

    audittrail(audit, "phase", "F1 axis - Post FT phase")
    post_ft_2df1(audit, "memory", "memory", p_in, f_in, f_out )

    audittrail(audit, "phase", "Final phase")
    tmpar = os.path.split( outputfilename )
    location = os.path.join( tmpar[0], 'tmp')
    post_2d(audit, "memory", outputfilename, p_in, f_in, f_out, location)

    audittrail(audit, "phase", "end of processing")
    
    #bidouille pour la phase
    spec_in = _f_in["javadata"]
    spec_out = _f_out["javadata"]
    if (p_in["autophase_2d"]):
        spec_in.getNNB_NMR_Spectrum().getAxisF(1).setPhaseOrder0(f_out["f1_autophase_0"])
        spec_in.getNNB_NMR_Spectrum().getAxisF(1).setPhaseOrder1(f_out["f1_autophase_1"])
        spec_in.getNNB_NMR_Spectrum().getAxisF(2).setPhaseOrder0(f_out["f2_autophase_0"])
        spec_in.getNNB_NMR_Spectrum().getAxisF(2).setPhaseOrder1(f_out["f2_autophase_1"])
        spec_out.getNNB_NMR_Spectrum().getAxisF(1).setPhaseOrder0(f_out["f1_autophase_0"])
        spec_out.getNNB_NMR_Spectrum().getAxisF(1).setPhaseOrder1(f_out["f1_autophase_1"])
        spec_out.getNNB_NMR_Spectrum().getAxisF(2).setPhaseOrder0(f_out["f2_autophase_0"])
        spec_out.getNNB_NMR_Spectrum().getAxisF(2).setPhaseOrder1(f_out["f2_autophase_1"])
        
    if (p_in["f1_phase"]):
        spec_in.getNNB_NMR_Spectrum().getAxisF(1).setPhaseOrder0(p_in["f1_phase_0"])
        spec_in.getNNB_NMR_Spectrum().getAxisF(1).setPhaseOrder1(p_in["f1_phase_1"])
        spec_out.getNNB_NMR_Spectrum().getAxisF(1).setPhaseOrder0(p_in["f1_phase_0"])
        spec_out.getNNB_NMR_Spectrum().getAxisF(1).setPhaseOrder1(p_in["f1_phase_1"])
        
    if (p_in["f1_autophase"]):
        spec_in.getNNB_NMR_Spectrum().getAxisF(1).setPhaseOrder0(f_out["f1_autophase_0"])
        spec_in.getNNB_NMR_Spectrum().getAxisF(1).setPhaseOrder1(f_out["f1_autophase_1"])
        spec_out.getNNB_NMR_Spectrum().getAxisF(1).setPhaseOrder0(f_out["f1_autophase_0"])
        spec_out.getNNB_NMR_Spectrum().getAxisF(1).setPhaseOrder1(f_out["f1_autophase_1"])
        
    if (p_in["f2_phase"]):
        spec_in.getNNB_NMR_Spectrum().getAxisF(2).setPhaseOrder0(p_in["f2_phase_0"])
        spec_in.getNNB_NMR_Spectrum().getAxisF(2).setPhaseOrder1(p_in["f2_phase_1"])
        spec_out.getNNB_NMR_Spectrum().getAxisF(2).setPhaseOrder0(p_in["f2_phase_0"])
        spec_out.getNNB_NMR_Spectrum().getAxisF(2).setPhaseOrder1(p_in["f2_phase_1"])
        
    if (p_in["f2_autophase"]):
        spec_in.getNNB_NMR_Spectrum().getAxisF(2).setPhaseOrder0(f_out["f2_autophase_0"])
        spec_in.getNNB_NMR_Spectrum().getAxisF(2).setPhaseOrder1(f_out["f2_autophase_1"]) 
        spec_out.getNNB_NMR_Spectrum().getAxisF(2).setPhaseOrder0(f_out["f2_autophase_0"])
        spec_out.getNNB_NMR_Spectrum().getAxisF(2).setPhaseOrder1(f_out["f2_autophase_1"]) 

    audittrail(audit, "phase", "2D Analysis")
    
    audittrail(audit, "phase", "Loading Parameters")
    

    audittrail( audit, "phase", "Peak picking and integration")
    processing = _p_in["javadata"]
    if (processing.isMakePeakPicking()):

    	audittrail( audit, "phase", "Peak picking phase")
        tmpar = os.path.split( outputfilename )
        peak_out = os.path.join( tmpar[0], "tmp", "peak.gtb")
    	pp_2d( audit, "memory", peak_out, p_in, f_in, f_out )

    	if (processing.isMakeIntegration()):
    		audittrail( audit, "phase", "Integration phase")
    		# pour l'instant il n'y a rien de fait

    audittrail( audit, "phase", "end of analysis")
    audittrail( audit, "close")

    # fermeture du fichier d'audit
    audit.close()


#---------------------------------------------------------------------------
# do_process_dosyf2
#---------------------------------------------------------------------------
def do_process_dosyf2(auditfile, _p_in, _f_in, _f_out, inputfilename, outputfilename):
    """
Do a half FT processing for DOSY, interface for the NMRNoteBook program:

    .toLowerCase()
arguments :
auditfile       the name of the audit file to create
filein          the name of the FID file
fileout         the name of the spectrum file
processing_in   the name of the java property list containing processing parameters 
spec_in         the name of the java property list containing fid details
spec_out        the name of the java property list containing final spectrum details
location        directory where to put projections.
    """

    # il faut supprimer l'action data check sans quoi on se retrouve
    # avec un jeu de donnee complexe
    _p_in["data_check"] = 0

    # ouverture du fichier d'audit
    audit = auditinitial(auditfile,"DOSY F2 FT processing")

    # Load parameters
    audittrail(audit, "phase", "Loading Parameters")
    
    audittrail(audit, "text", "open FID ")
    f_in = todict(_f_in)
    
    audittrail(audit, "text", "open processing parameters ")
    p_in = todict(_p_in)
    p_in["select_state"] = 1
    p_in["f1_state"] = "real"
    if ( p_in["modulus"] ):
        p_in["f2_state"] = "real"
    else:
        p_in["f2_state"] = "complex"
    
    audittrail(audit, "text", "open Spectra ")
    f_out = todict(_f_out)

    # DOSY processing via NPK entry point
    PreDosy2D(audit,p_in,f_in,f_out,inputfilename,outputfilename)

    # phasing transfert
    spec_in = _f_in["javadata"]
    spec_out = _f_out["javadata"]
    if (p_in["f2_phase"]):
        spec_in.getNNB_NMR_Spectrum().getAxisF(2).setPhaseOrder0(p_in["f2_phase_0"])
        spec_in.getNNB_NMR_Spectrum().getAxisF(2).setPhaseOrder1(p_in["f2_phase_1"])
        spec_out.getNNB_NMR_Spectrum().getAxisF(2).setPhaseOrder0(p_in["f2_phase_0"])
        spec_out.getNNB_NMR_Spectrum().getAxisF(2).setPhaseOrder1(p_in["f2_phase_1"])
        
    if (p_in["f2_autophase"]):
        spec_in.getNNB_NMR_Spectrum().getAxisF(2).setPhaseOrder0(f_out["f2_autophase_0"])
        spec_in.getNNB_NMR_Spectrum().getAxisF(2).setPhaseOrder1(f_out["f2_autophase_1"]) 
        spec_out.getNNB_NMR_Spectrum().getAxisF(2).setPhaseOrder0(f_out["f2_autophase_0"])
        spec_out.getNNB_NMR_Spectrum().getAxisF(2).setPhaseOrder1(f_out["f2_autophase_1"]) 


    # end of processing
    audittrail( audit, "phase", "end of processing")
    audittrail( audit, "close")

    # fermeture du fichier d'audit
    audit.close()

#---------------------------------------------------------------------------
# do_process_dosyILT
#---------------------------------------------------------------------------
def do_process_dosyILT(auditfile, _p_in, _f_in, _f_out, inputfilename, outputfilename):
    """
Do ILT processing for DOSY, interface for the NMRNoteBook program:

    
arguments :
auditfile       the name of the audit file to create
filein          the name of the FID file
fileout         the name of the spectrum file
processing_in   the name of the java property list containing processing parameters 
spec_in         the name of the java property list containing fid details
spec_out        the name of the java property list containing final spectrum details
location        directory where to put projections.
    """

    # ouverture du fichier d'audit
    audit = auditinitial(auditfile,"DOSY F1 Inverse Laplace Transform (ILT) processing")

    # Load parameters
    audittrail(audit, "phase", "Loading Parameters")
    
    audittrail(audit, "text", "open FID ")
    f_in = todict(_f_in)
    
    audittrail(audit, "text", "open processing parameters ")
#    dict_out(_p_in,"before todict")    # modif MAD dec 2008
    p_in = todict(_p_in)
    
    audittrail(audit, "text", "open Spectra ")
    f_out = todict(_f_out)

#    dict_out(p_in,"before DOSY2D call")    # modif MAD dec 2008

    # DOSY processing via NPK entry point
#    p_in["dump_preset"]=1    # modif MAD dec 2008
    tmpar = os.path.split( outputfilename )
    location = os.path.join( tmpar[0], 'tmp')
    Dosy2D(audit,p_in,f_in,f_out,inputfilename,outputfilename, location)

    # end of processing
    audittrail( audit, "phase", "end of processing")
    audittrail( audit, "close")

    # fermeture du fichier d'audit
    audit.close()

#---------------------------------------------------------------------------
# do_process_tosyILT
#---------------------------------------------------------------------------
def do_process_tosyILT(auditfile, _p_in, _f_in, _f_out, inputfilename, outputfilename):
    """
Do ILT processing for TOSY, interface for the NMRNoteBook program:

    
arguments :
auditfile       the name of the audit file to create
filein          the name of the FID file
fileout         the name of the spectrum file
processing_in   the name of the java property list containing processing parameters 
spec_in         the name of the java property list containing fid details
spec_out        the name of the java property list containing final spectrum details
location        directory where to put projections.
    """

    # ouverture du fichier d'audit
    audit = auditinitial(auditfile,"DOSY F1 Inverse Laplace Transform (ILT) processing")

    # Load parameters
    audittrail(audit, "phase", "Loading Parameters")
    
    audittrail(audit, "text", "open FID ")
    f_in = todict(_f_in)
    
    audittrail(audit, "text", "open processing parameters ")
#    dict_out(_p_in,"before todict")    # modif MAD dec 2008
    p_in = todict(_p_in)
    
    audittrail(audit, "text", "open Spectra ")
    f_out = todict(_f_out)

#    dict_out(p_in,"before DOSY2D call")    # modif MAD dec 2008

    # DOSY processing via NPK entry point
#    p_in["dump_preset"]=1    # modif MAD dec 2008
    Dosy2D(audit,p_in,f_in,f_out,inputfilename,outputfilename)

    # end of processing
    audittrail( audit, "phase", "end of processing")
    audittrail( audit, "close")

    # fermeture du fichier d'audit
    audit.close()

#---------------------------------------------------------------------------
# do_process_maxent1d
#---------------------------------------------------------------------------
def do_process_maxent1d(auditfile, _p_in, _f_in, _f_out, inputfilename, outputfilename):
    """
Do Maximum Entropy processing, interface for the NMRNoteBook program:

    
arguments :
auditfile       the name of the audit file to create
filein          the name of the FID file
fileout         the name of the spectrum file
processing_in   the name of the java property list containing processing parameters 
spec_in         the name of the java property list containing fid details
spec_out        the name of the java property list containing final spectrum details
location        directory where to put projections.
    """

    # ouverture du fichier d'audit
    audit = auditinitial(auditfile,"1D maximum entropy processing")

    # Load parameters
    audittrail(audit, "phase", "Loading Parameters")
    
    audittrail(audit, "text", "open FID ")
    f_in = todict(_f_in)
    
    audittrail(audit, "text", "open processing parameters ")
    p_in = todict(_p_in)
    
    audittrail(audit, "text", "open Spectra ")
    f_out = todict(_f_out)

    # DOSY processing via NPK entry point
    MaxEnt1D(audit,p_in,f_in,f_out,inputfilename,outputfilename)

    # end of processing
    audittrail( audit, "phase", "end of processing")
    audittrail( audit, "close")

    # fermeture du fichier d'audit
    audit.close()

#---------------------------------------------------------------------------
# do_process_maxent2d
#---------------------------------------------------------------------------
def do_process_maxent2d(auditfile, _p_in, _f_in, _f_out, inputfilename, outputfilename):
    """
Do Maximum Entropy processing, interface for the NMRNoteBook program:

    
arguments :
auditfile       the name of the audit file to create
filein          the name of the FID file
fileout         the name of the spectrum file
processing_in   the name of the java property list containing processing parameters 
spec_in         the name of the java property list containing fid details
spec_out        the name of the java property list containing final spectrum details
location        directory where to put projections.
    """

    # ouverture du fichier d'audit
    audit = auditinitial(auditfile,"2D Maximum Entropy processing")

    # Load parameters
    audittrail(audit, "phase", "Loading Parameters")
    
    audittrail(audit, "text", "open FID ")
    f_in = todict(_f_in)
    
    audittrail(audit, "text", "open processing parameters ")
    p_in = todict(_p_in)
    
    audittrail(audit, "text", "open Spectra ")
    f_out = todict(_f_out)

    # DOSY processing via NPK entry point
    MaxEnt2D(audit,p_in,f_in,f_out,inputfilename,outputfilename)

    # end of processing
    audittrail( audit, "phase", "end of processing")
    audittrail( audit, "close")

    # fermeture du fichier d'audit
    audit.close()

