#!/usr/bin/env python
# encoding: utf-8
"""
Library for NNB utilities
"""


import java
import javax.swing as swing
import java.awt as awt
import sys
import traceback
import nmrtec.nmrNotebook.gui
import nmrtec.nmrNotebook.model.annotation.annotation2D.NNB_Distance2D
import nmrtec.nmrNotebook.controller.Controller as Cont


class DialogText:
    """
    script python qui ouvre une fenetre de dialogue avec un texte selectionnable
    pour appeler le script 
    DialogText (titre, label, texte)
    par exemple : DialogText ("Voici vos resultats","Ligne 1 \n ligne 2\n \n \n \n \n \n  ligne n \n")
    """
    def __init__(self,label,text):
        self.dialo = swing.JFrame()
        self.dialo.setTitle("NMRNoteBook")
        self.dialo.setIconImage(nmrtec.nmrNotebook.gui.ToolKit.getImage("Icon_Logo"))  #modif RH 06-01-2009
        panel = swing.JPanel()
        panel.setLayout(awt.GridBagLayout())  #modif RH  18-11-2008
        lab = swing.JLabel(label)
        gc = awt.GridBagConstraints(0,0,1,1,1.0,0.0,awt.GridBagConstraints.NORTHWEST,awt.GridBagConstraints.HORIZONTAL,awt.Insets(2,2,2,2),0,0)
        panel.add(lab,gc)
        text_a = swing.JTextArea(text)
        scrollp = swing.JScrollPane(text_a)
        scrollp.setMinimumSize(awt.Dimension(200,200))

        gc = awt.GridBagConstraints(0,1,2,1,1.0,1.0,awt.GridBagConstraints.NORTHWEST,awt.GridBagConstraints.BOTH,awt.Insets(2,2,2,2),0,0)
        panel.add(scrollp,gc)
       
        text_a.setEditable(0)       # On ne peut pas ecrire dans le Text Area
        close_but = swing.JButton("close")    
        close_but.actionPerformed = self.exit
        gc = awt.GridBagConstraints(1,2,1,0,0.0,0.0,awt.GridBagConstraints.NORTHWEST,awt.GridBagConstraints.HORIZONTAL,awt.Insets(2,2,2,2),0,0)
        panel.add(close_but,gc)
        self.dialo.getContentPane().add(panel)
        
        self.dialo.setSize(400,300)
        self.dialo.setResizable(1)
        self.dialo.setLocationRelativeTo(None)
        #self.dialo.setAlwaysOnTop(1)  # RH  16-12-2008
        self.dialo.show()
    def exit(self,e):
        self.dialo.dispose()

def DialogValue(label,defvalue="",ValidEntries=None):
    """
    creates a modal dialog box allowing to get a value from the user
    defvalue is an optionnal default value
    ValidEntries : if not None, defines the list of valid entries; user will have to choose in a ComboBox
    
    return None is Canceled
    """
    return swing.JOptionPane.showInputDialog(None,label,"NMRNoteBook",swing.JOptionPane.QUESTION_MESSAGE,None,ValidEntries,defvalue)

def alert(label):
    """
    creates a modal dialog box allowing to alert user about something specific
    label is the text in the box
    
    """
    return swing.JOptionPane.showMessageDialog(None,label,"warning", swing.JOptionPane.WARNING_MESSAGE) 
    
def listOfRectangles():
    """
    returns the list of defined rectangle in the current 2D dataset
    each entry is 4-tuple : (f1_left, f2_left, f1_right, f2_right)
    """
    CDD = Cont.getCurrentControllerDeDocument()
    spectre = CDD.getCurrentNMR_Folder()
    annot = spectre.getListAllAnnotations()
    tt = []
    for i in range( len(annot)):
        a =  annot[i]
    #    print i, a
        if isinstance(a, nmrtec.nmrNotebook.model.annotation.annotation2D.NNB_Rectangle2D):
            llx=a.getLLPoint().getValueF2().getValue()
            lly=a.getLLPoint().getValueF1().getValue()
            urx=a.getURPoint().getValueF2().getValue()
            ury=a.getURPoint().getValueF1().getValue()
            f1_left = min(round(lly),round(ury))
            f2_left = min(round(llx),round(urx))
            f1_right = max(round(lly),round(ury))
            f2_right = max(round(llx),round(urx))
            tt.append((  f1_left, f2_left, f1_right, f2_right))
    return tt

def listOfLines():
    """
    returns the list of defined Lines in the current 2D dataset
    each entry is 5-tuple : (f1_left, f2_left, f1_right, f2_right, orientation)
    where orientation is : -1 for horizontal ; 1 for vertical
    """
    CDD = Cont.getCurrentControllerDeDocument()
    spectre = CDD.getCurrentNMR_Folder()
    annot = spectre.getListAllAnnotations()
    tt = []
    for i in range( len(annot)):
        a =  annot[i]
    #    print i, a
        if isinstance(a,nmrtec.nmrNotebook.model.annotation.annotation2D.NNB_Line2D):
            orientation = a.getOrientation()    # horizontal is -1; vertical is 1
            llx=a.getPoint1().getValueF2().getValue()
            lly=a.getPoint1().getValueF1().getValue()
            urx=a.getPoint2().getValueF2().getValue()
            ury=a.getPoint2().getValueF1().getValue()
            f1_left = min(round(lly),round(ury))
            f2_left = min(round(llx),round(urx))
            f1_right = max(round(lly),round(ury))
            f2_right = max(round(llx),round(urx))
            tt.append((  f1_left, f2_left, f1_right, f2_right, orientation))
    return tt 
    
def listOfDistances():
    """
    returns the list of defined Distances in the current 2D dataset
    each entry is 5-tuple : (f1_left, f2_left, f1_right, f2_right, orientation)
    where orientation is : -1 for horizontal ; 1 for vertical
    """
    CDD = Cont.getCurrentControllerDeDocument()
    spectre = CDD.getCurrentNMR_Folder()
    annot = spectre.getListAllAnnotations()
    tt = []
    for i in range( len(annot)):
        a =  annot[i]
    #    print i, a
        if isinstance(a,nmrtec.nmrNotebook.model.annotation.annotation2D.NNB_Distance2D):
            orientation = a.getOrientation()    # horizontal is -1; vertical is 1
            llx=a.getPoint1().getValueF2().getValue()
            lly=a.getPoint1().getValueF1().getValue()
            urx=a.getPoint2().getValueF2().getValue()
            ury=a.getPoint2().getValueF1().getValue()
            f1_left = min(round(lly),round(ury))
            f2_left = min(round(llx),round(urx))
            f1_right = max(round(lly),round(ury))
            f2_right = max(round(llx),round(urx))
            tt.append((  f1_left, f2_left, f1_right, f2_right, orientation))
    return tt 
    
def listOfNotes():
    """
    returns the list of defined Notes in the current 2D dataset
    each entry is 3-tuple : ( f1, f2, text)
    
    """
    CDD = Cont.getCurrentControllerDeDocument()
    spectre = CDD.getCurrentNMR_Folder()
    annot = spectre.getListAllAnnotations()
    tt = []
    for i in range( len(annot)):
        a =  annot[i]
        if isinstance(a,nmrtec.nmrNotebook.model.annotation.annotation2D.NNB_Note2D):
            text = a.getPointText().getNNB_Text()
            llx = a.getPointText().getValueF2().getValue()
            lly = a.getPointText().getValueF1().getValue()
            tt.append((  round(lly), round(llx), text))
    return tt 
    
    
startDirDef="~/Documents/"
def DialogFile(label,type="FILE",startDir=None):
    """type is one of    DIRECTORY / FILE / BOTH
    """
    global startDirDef
    if startDir == None:
        startDir = startDirDef
    frame = swing.JFrame() #modif RH 06-01-2009
    frame.setIconImage(nmrtec.nmrNotebook.gui.ToolKit.getImage("Icon_Logo"))  #modif RH 06-01-2009
    folcho = swing.JFileChooser(startDir)
    folcho.setDialogTitle(label)
    if type == "DIRECTORY":
        folcho.setFileSelectionMode(swing.JFileChooser.DIRECTORIES_ONLY)
    elif type == "FILE":
        folcho.setFileSelectionMode(swing.JFileChooser.FILES_ONLY)
    elif type == "BOTH":
        folcho.setFileSelectionMode(swing.JFileChooser.FILES_AND_DIRECTORIES)
    else:
        raise("Should never get there")
    #value = folcho.showOpenDialog(None) #modif RH 06-01-2009
    folcho.showOpenDialog(frame) #modif RH 06-01-2009
    startDirDef = folcho.getCurrentDirectory()
    return folcho.selectedFile
    
def formatExceptionInfo(maxTBlevel=5):
    """
    This module formats the information coming from the exception and makes it reachable in an alert box
    
    """
    cla, exc, trbk = sys.exc_info()
    excName = cla.__name__
    try:
        excArgs = exc.__dict__["args"]
    except KeyError:
        excArgs = "<no args>"
    excTb = traceback.format_tb(trbk, maxTBlevel)
    #print "Traceback: \n%s %s: %s" %(excTb[0], excName, excArgs[0])# modif RH 21012009
    #return (excName, excArgs, excTb)  # modif RH 21012009
    return ("Traceback (innermost last): \n%s %s: %s" %(excTb[0], excName, excArgs[0]))# modif RH 21012009