"""
 The library of NMR processing functions to be used with the NPK program.

This library implement the standard functions needed for NMR processing.
Most of theese functions require that the NPK mathematical kernel is loaded.

"""

__author__ = "Franck Thomas <franck.thomas@utbm.fr> and Vincent Catherinot <v.catherinot@nmrtec.com>"
__date__ = "Oct 2007"

try:
    from nmrtec.nmrNotebook.kernel.Kore import *
except ImportError:
    print "The NPK dll seems to be lacking"

def AujInit(n1,n2):
        """
        Prepare the AUJ analysis of a multiplet in window (n1,n2)
        """
	dim(1)

        # get dataset parameters
        sw0 = get_specw_1d()
        sz0 = get_si1_1d()
        offset0 = get_offset_1d()
        resol = sw0/sz0

        extract(n1,n2)
        sz1 = get_si1_1d()
        offset1 = get_offset_1d()
        s1 = 0
        s2 = 0 

        for i in range(1,sz1,2):
                y = val1d(i)
                s1 = s1 + y
                s2 = s2 + y * i

        g = 1 + 2 * int((s2 / s1 - 1) / 2)
        print( repr(sz1) )
        print( repr(g) )

        # compute size
        sz2= 4 * power2( sz1 )
        if sz2 < 1024 :
                sz2 = 1024      
        print( repr(sz2) )

        szi = sz2 / 2 + g -1
        chsize ( szi )
        reverse()
        chsize (sz2)
        reverse()

        offset ( offset1 - resol * (szi - sz1))
        specw ( resol * sz2 )

        real()
        iftbis()
        revf ()

