"""
This class extend the standard dictionnary behaviour.
It is used to read and store all the parameters used by the NPK program.

- it adds the possibility to load from files and dump/store to files the content of the dictionnary
  the file will be of the form :
  key=value, with one entry per line

- when extracted from the dictionnnary, values are interpreted in several manner
    o 1K is replaced by 1024 and all multiples (4k; 24k; etc...)
    o 1M is replaced by 1024*1024 and all multiples (4M; 24M; etc...)
    o all arithmetic is evaluated : math.pi/2, 0.1*get_si1_2d(), etc...
  
"""

# UserDict is used for 2.1 compatibility required by jython
import UserDict
import os

#---------------------------------------------------------------------------
def parse(line):
    import re
    """Parse one property entry and prepare it to be entered in the  dictionnary
    """
    l = re.split(r'(?<!\\)=', line,1)   #matches = but not \=
#    dkey = re.sub(r'\\=', '=',l[0]).lower()
    dkey = re.sub(r'\\=', '=',l[0])
    val = re.sub(r'\\=', '=',l[1])      # replaces \= by =
    
#    l = line.strip().split("=",2)
#    dkey=l[0].lower()
#    val = l[1]

#    fval=NPKevaluate(val)
#    print dkey,fval
    return (dkey,val)

#---------------------------------------------------------------------------
def NPKevaluate(val):
    """
    - val is interpreted in several manner
        o 1K is replaced by 1024 and all multiples (4k; 24k; etc...)
        o 1M is replaced by 1024*1024 and all multiples (4M; 24M; etc...)
        o all arithmetic is evaluated : math.pi/2, 0.1*get_si1_2d(), etc...
    """
    import math
    from NPK import *
    found=0
#    print "%i %s : " % (get_itype_1d(),val),
    try:    # try eval()
        fval=eval(val)
        if callable(fval):  # we want to avoid functions !
            pass
#            print " callable"
        else:
#            print " worked"
            found=1
    except:
#        print " failed"
        pass
    if (not found):
        try:    # try to type value
            if ( val.endswith("k")):
                fval = 1024*int(val[0:len(val)-1])
                found=1
            elif ( val.endswith("M")):
                fval = 1024*1024*int(val[0:len(val)-1])
                found=1
            else:
                fval = float(val)
                found=1
        except:
            pass
    if (not found):
        try:    # try to type value
            if ( val.lower() == "false" ):
                fval = 0
                found=1
            elif ( val.lower() == "true" ):
                fval = 1
                found=1
        except:
            fval = val
            found=1
    if (not found):
        fval=val
    return fval


#---------------------------------------------------------------------------
class NPKParam(UserDict.UserDict):
    "doc..."
    #---------------------------------------------------------------------------
    def __str__(self):
        return 'parameters :\n'+repr(self)

    #---------------------------------------------------------------------------
    def __getitem__(self,key):
        """overloading of the dictionnary [] method
        evaluates the content
        """
        i=NPKevaluate(self.data[key])
        return i

    #---------------------------------------------------------------------------
    def raw(self,key):
        """
        acces to the raw data present in the diictionnary, necessary in certain cases
        """
        return self.data[key]

    #---------------------------------------------------------------------------
    def dump(self,fname):
        """
        dump the content of the Parameter dictionnary as a property list file
        entries are not evaluated
        
        one entry per line with the following syntax :
        entry=value
        
        """
        try:
            fout = open(fname, 'w')
        except:
            print "Error while opening file :", sys.exc_info()[0]
            raise

        fout.write("#Property list file, dumped :"+time.strftime("%a, %d %b %Y %H:%M:%S %Z", time.localtime())+"\n")

        for i in self.keys():
            lk = re.sub('=', r'\\=', i)      # replaces = by \=
            lv = re.sub('=', r'\\=', str(self.data[i]))      # replaces = by \=

            fout.write(lk+"="+lv+"\n")
        fout.close()

    #---------------------------------------------------------------------------
    def store(self,fname):
        """
        write the content of the Parameter dictionnary as a property list file
        equivalent to dump, BUT entries are evaluated
        
        one entry per line with the following syntax :
        entry=value
        
        """
        try:
            fout = open(fname, 'w')
        except:
            print "Error while opening file :", sys.exc_info()[0]
            raise

        fout.write("#Property list file, output :"+time.strftime("%a, %d %b %Y %H:%M:%S %Z", time.localtime())+"\n")

        for i in self.keys():
            lk = re.sub('=', r'\\=', i)      # replaces = by \=
            lv = re.sub('=', r'\\=', self[i])      # replaces = by \=

            fout.write(lk+"="+lv+"\n")
        fout.close()

    #---------------------------------------------------------------------------
    def load(self,fname):
        """
        load a property list file as a dictionary
        
        one entry per line with the following syntax :
        entry=value
        
        keys are set to lowercase
        """
        try:
            fin = open(fname)
            f = fin.read()
            ls = f.split("\n")
            for v in ls:
                if ( v == "" or v[0] == "#"):   # skip comments and empty lines
                    pass
                else:
                    (dkey,fval) = parse(v.strip())
                    try:            # check key existence
                        val=self.data[dkey]
                    except:
                        pass
                    else:
                        print "WARNING, key -",dkey,"- defined twice in",fname
                        print "         previous value :",val," new value",fval
                    self.data[dkey]=fval
            fin.close
        except:
            print "File "+fname+" not found."
            print "Creating empty parameter list\n"

    #---------------------------------------------------------------------------
    def build_default(self,default_list):
        """ build the default parameter dictionary
        
        used in standard actions,
        returns a dictionary built from the default parmaters (see do_default.py and Param/*)
        and the additional parameters defined in the optionnal p_in_arg overwrite the default values
        
        """
        from NPK import Generic
        base = os.path.join(Generic.get_npk_path(),"Param")
        for l in default_list:
            self.load( os.path.join(base,l) +".gtb" )
#            for i in pp.keys():
#                self[i]=pp[i]

# if launched with
# NPK -e Param.py 

if (__name__ == "__main__"):
  import math
  p=NPKParam()
  p["a"]="pa"
  p["b"]="1k"
  p["c"]="math.pi"
  p["d"]="2*3*4"
  for i in p.keys():
      print i,":",p[i],type(p[i])
