import tempfile
import math
#from nmrtec.nmrNotebook.kernel.Kore import *

class axis:
    """ This class describes the parameter of a pea along one axis
    """
    
    def __init__(self,f=0.0,w=0.0,p=0.0,fe=0.0,we=0.0,pe=0.0):
        self.__frequency=f
        self.__frequencyError=fe
        self.__width=w
        self.__widthError=we
        self.__phase=p
        self.__phaseError=pe

    def __str__(self):
        return ("freq : %f +/- %f width : %f +/- %f phase : %f +/- %f"
            % (self.__frequency,self.__frequencyError,self.__width,self.__widthError,self.__phase,self.__phaseError))

    def set_frequency(self,value):
        """ set the value of the frequency of the peak along the axis"""
        self.__frequency=value

    def get_frequency(self):
        """ get the value of the frequency of the peak along the axis"""
        return self.__frequency

    def set_frequencyError(self,value):
        """ set the value of the error on the frequency of the peak along the axis
            error is always positive"""
        self.__frequencyError=abs(value)

    def get_frequencyError(self):
        """ get the value of the error on the frequency of the peak along the axis"""
        return self.__frequencyError

    def set_width(self,value):
        """ set the value of the width of the peak along the axis
            width is always positive"""
        self.__width=abs(value)

    def get_width(self):
        """ get the value of the width of the peak along the axis
            width is always positive"""
        return self.__width

    def set_widthError(self,value):
        """ set the value of the error on the width of the peak along the axis
            error is always positive"""
        self.__widthError=abs(value)

    def get_widthError(self):
        """ get the value of the error on the width of the peak along the axis
            width is always positive"""
        return self.__widthError

    def set_phase(self,value):
        """ set the value of the phase of the peak along the axis
            phase is contained in -180<phase<=180 """
        v=value
        while v<-180.0:
            v=v+360.0
        while v>=180.0:
            v=v-360.0
        self.__phase=abs(v)

    def get_phase(self):
        """ get the value of the phase of the peak along the axis"""
        return self.__phase

    def set_phaseError(self,value):
        """ set the value of the error on the phase of the peak along the axis
            error is always positive"""
        self.__phaseError=abs(value)
    
    def get_phaseError(self):
        """ get the value of the phase of the peak along the axis"""
        return self.__phaseError


class Peak:
    def __init__(self,amplitude=0.0, amplitudeError=0.0):
        self.__amplitude = amplitude
        self.__amplitudeError = amplitudeError
        self.__annotation = ""

    def set_amplitude(self,value):
        """ set the value of the amplitude of the peak"""
        self.__amplitude=value

    def get_amplitude(self):
        """ get the value of the amplitude of the peak"""
        return self.__amplitude

    def set_annotation(self,value):
        """ set the annotation for the peak """
        self.__annotation=str(value)

    def get_annotation(self):
        """ set the annotation for the peak """
        return self.__annotation

    def set_amplitudeError(self,value):
        """ set the value of the error on the amplitude of the peak 
            error is always positive"""
        self.__amplitudeError=abs(value)

    def get_amplitudeError(self):
        """ get the value of the error on the amplitude of the peak"""
        return self.__amplitudeError

class Peak1D(Peak):
    def __init__(self,a=0.0,ae=0.0,f=0.0,fe=0.0,w=0.0,we=0.0,p=0.0,pe=0.0):
        self.__amplitude = a
        self.__amplitudeError = ae
        self.F1 = axis(f,fe,w,we,p,pe)
    def __str__(self):
        return ("Ampl : %f +/- %f "%(self.__amplitude,self.__amplitudeError) + " F1: " + str(self.F1))
    def get_volume(self):
        """ get the value ofvolume of the peak  """
        return self.__amplitude*self.F1.__width

class Peak2D(Peak):
    def __init__(self,a=0.0,ae=0.0,f=0.0,fe=0.0,w=0.0,we=0.0,p=0.0,pe=0.0):
        self.__amplitude = a
        self.__amplitudeError = ae
        self.F1 = axis()
        self.F2 = axis()
    def __str__(self):
        return ("Ampl : %f +/- %f "%(self.__amplitude,self.__amplitudeError) + " F1: " + str(self.F1) + " F2: " + str(self.F2))
    def get_volume(self):
        """ get the value ofvolume of the peak  """
        return self.__amplitude*self.F1.__width*self.F2.__width

class Peak3D(Peak):
    def __init__(self,a=0.0,ae=0.0,f=0.0,fe=0.0,w=0.0,we=0.0,p=0.0,pe=0.0):
        self.__amplitude = a
        self.__amplitudeError = ae
        self.F1 = axis()
        self.F2 = axis()
        self.F3 = axis()
    def __str__(self):
        return ("Ampl : %f +/- %f "%(self.__amplitude,self.__amplitudeError) + " F1: " + str(self.F1) + " F2: " + str(self.F2) + " F3: " + str(self.F3))
    def get_volume(self):
        """ get the value ofvolume of the peak  """
        return self.__amplitude*self.F1.__width*self.F2.__width*self.F3.__width

p=Peak3D()
print p
