"""
    Utility for handling Varian files
	
	this one is debuged yet !!
"""
__author__ = "Marc A. Delsuc <ma.delsuc@cbs.cnrs.fr>"
__date__ = "Oct 2006"

import struct
import re
import os
import time

try:
    import nmrtec.nmrNotebook.kernel.Kore as K
except ImportError:
    print "The NPK dll seems to be lacking"

################################################################
def find_procpar(dir="."):
    """
    find the procpar file associated to the directory dir and return its name
    
    """
    import os.path as op

# search acqu param file
    found = 0
    l = ('procpar','PROCPAR')
    for a in l:
        filename=(op.join(dir,a))
        try:    # check all versions of acqu file
            f = open(filename)
            f.close()
            found = 1
            break
        except:
            print a+" not found"
    if (found == 0):
        raise "No procpar file found in "+dir
    return(filename)


################################################################
def read_param(filename="procpar"):
    """ 
    load a Varian procpar file as a dictionnary
    
    arrayed values are stored in python array
    
    comments (lines starting with $$) are stored in the special entrey [comments]
    
    M-A Delsuc oct 2006
    """
    try:
        fin = open(filename)
    except:
        raise filename," cannot be accessed"
    # read file
    dict = {}
    f=fin.read()
    fin.close()
    ls= f.split("\n")

    while ls:
        v=ls.pop(0)
        v = v.lstrip()
        m=re.match(r"([a-zA-Z]\w*)",v) # match entry name
        if (m):
            key = m.group(1)
            v=ls.pop(0)
            mm=re.match(r"(\d+) (.*)$",v )   # match arrays
            (numb,line)=mm.group(1,2)
            numb = int(numb)
            if (re.match(r'"',line)):
                while (not re.match(r'".*"$',line)):  # glue lines with return in it
                    v=ls.pop(0)
                    line = line + v
            if (numb == 1):
                dict[key]=line
            else:
                print numb
                while (numb > len(line.split())):   # if entry remains
                    v=ls.pop(0)
                    line = line + " " + v
                array=line.split()
                if ((int(numb)) != len(array)):
                        raise "size mismatch in array"
                dict[key] = array
            print key,"=",dict[key]
            ls.pop(0) 
        else:
            raise "Varian wrong format"
# debug code
#    for i in dict.keys():
#        print i+" = "+str(dict[i])
    return dict

################################################################
def read_1D(size,filename="fid"):
    """
    Reads in a Varian 1D fid
    
    size is the number of data-points in the fid
    uses java low level file access
    """
    from java.io import RandomAccessFile
#    from nmrtec.util.IO import SwapByte
    import jarray

# read binary
    K.dim(1)
    K.chsize(int(size))
    f=RandomAccessFile(filename,"r")
    for i in range(size):
        x = f.readInt()
#        x = SwapByte.swap(x)       # uncoment if swapbyte is required
#        print i,x
        K.setval(i+1,float(x))      # copy to 1D buffer

################################################################
def read_2D(sizeF1,sizeF2,filename="ser"):
    """
    Reads in a Bruker 2D fid
    
    sizeF1 is the number of fid
    sizeF2 is the number of data-points in the fid
    uses java low level file access
    """
    from java.io import RandomAccessFile
#    from nmrtec.util.IO import SwapByte
    import jarray

# read binary
    K.dim(2)
    K.chsize(int(sizeF1),int(sizeF2))
    f=RandomAccessFile(filename,"r")
    for i1 in range(sizeF1):
        for i2 in range(sizeF2):
            x = f.readInt()
#        x = SwapByte.swap(x)       # uncoment if swapbyte is required
            K.setval(i1+1,i2+1,float(x))      # copy to 1D buffer

################################################################
def Import_1D(filename="fid",outfile=""):
    """
    Imports a 1D Bruker fid
    
    """
    import os.path as op
    import NPK.Generic
    global zerotimeposition

    if (not op.exists(filename)):
        raise filename+" : file not found"
    dir=op.dirname(filename)
    acqu = read_param(find_acqu(dir))
    size= int(acqu['$TD'])  # get size
    print "importing 1D FID, size =",size
    read_1D(size,filename)

# then set parameters
    K.specw(float(acqu['$SW_h']))
    K.freq(float(acqu['$SFO1']),float(acqu['$SFO1']))

    proc = read_param(find_proc(dir))
    K.offset( offset(acqu, proc) )
    
    if (outfile != ""):
        K.dim(1)
        K.writec(outfile)
        K.join(outfile)
        K.putheader("zerotimeposition", repr(zerotimeposition))
        K.disjoin()
        param={}
        param['axisf1_zerotimeposition'] = zerotimeposition
        NPK.Generic.dict_dump(param,outfile+'.gtb')
    
################################################################
def Import_2D(filename="ser",outfile=""):
    """
    Imports a 2D Bruker ser
    
    """
    import os.path as op

    if (not op.exists(filename)):
        raise filename+" : file not found"
    dir=op.dirname(filename)
    acqu = read_param(find_acqu(dir))
    acqu2 = read_param(find_acqu2(dir))
    sizeF1= int(acqu2['$TD'])  # get size
    sizeF2= int(acqu['$TD'])  # get size
    print "importing 2D FID, size =",sizeF1,"x",sizeF2
    read_2D(sizeF1,sizeF2,filename)

# then set parameters
    K.specw(float(acqu2['$SW_h']), float(acqu['$SW_h']))
    K.freq(float(acqu['$SFO1']), float(acqu2['$SFO1']), float(acqu['$SFO1']))

    proc = read_param(find_proc(dir))
    proc2 = read_param(find_proc2(dir))
    K.offset( offset(acqu2, proc2), offset(acqu, proc))

    zerotimeposition = zerotime(acqu)
    if (outfile != ""):
        K.dim(2)
        K.writec(outfile)
        K.putheader("zerotimeposition",zerotimeposition)
    
    
    

