"""
    batch_dosy
    
    This utility is meant for starting a DOSY process on a remote cluster
    
    First prepare every thing for running, in a parameter files, say param.gtb
    
    Then dump the processing parameters with the actions:
        dump_preset ( dump all the parameters to a .gtb file)
        execute_ilt (realizes or not the ilt operation)
    
    for instance : 
    NPK file.gs2 dosy.gs2  param.gtb  dump_preset=1  dump_preset_file=preset.gtb  execute_ilt=0
    
    then starts the processing in batch with:
    
    python batch_dosy.py file.gs2 dosy.gs2 preset.gtb { N_JOBS=8 }
    or
    NPK -e batch_dosy.py file.gs2 dosy.gs2 preset.gtb { N_JOBS=8 }

    with N_JOBS=x being an optionnal parameter
    
"""

# parameters to adapt :
""" the command which launch NPK on the machine"""
NPK="~/bin/NPK"


"""the command which launch a job on the cluster
launch will be :

PREBATCH  NPK paramemters  POSTBATCH

"""
# the following will work on a PBS system
PREBATCH="echo"
POSTBATCH=" | qsub  -N batch_dosy -q serial_24h -d $PWD"
# -q queue_name : put your own here; 

## the following will work on a simple unix (linux or MacOsX) multiprocessor desktop system
#PREBATCH=" "
#POSTBATCH="&"


""" the number of jobs which will be launched.
If BATCH is some kind of queing system, (for instance a cluster running "qsub")
   it is a good idea to have here more submitted jobs than processors available on the cluster
   then a large number is ok.
On the other hand
If BATCH starts directly a job (for instance on a multiprocessor desktop)
   it is a good idea to have here less jobs than processors available on the machine
   then a number equal to the number of processors (or slightly less) is fine.
"""
N_JOBS=8

"""
After launching the different jobs, the process waits for all processings to be finished
It spools the directory with the frequency WAIT_DELAY for checking output files
"""
WAIT_DELAY = 10


#########################################################################################
def split_list(list,n=N_JOBS):
    """ takes one list and make n sublist out of it"""
    list_of_list=[]
    for i in range(n):
        list_of_list.append([])
    i = 0
    for l in list:
        list_of_list[i].append(l)
        i=i+1
        if (i>=n):
            i=0
    return list_of_list

#########################################################################################
def start_batch(input,output,param,nchunk):
    from Generic import dict_load, dict_dump
    import os, shutil
    outputdir=os.path.dirname(output)
# load param
    p_in=dict_load(param)
# cut list
    ll=split_list( eval(p_in["col_list"]),nchunk)
    outputfiles=[]
#prepare files
    for i in range(nchunk):
        ldir="ldir" + str(i)
        infile = os.path.join(ldir,input)
        try:
            os.mkdir(ldir)
        except:
            pass
        shutil.copy(input,infile)
        paramfile=os.path.join(ldir,param)
        outfile=os.path.join(ldir,output)

#        paramfile=param.replace(".gtb","_"+str(i)+".gtb")
#        outfile=output.replace(".gs2","_"+str(i)+".gs2")
        outputfiles.append(outfile)
        p=p_in.copy()
        p["col_list"]=ll[i]
        dict_dump(p,paramfile)
        launch('DOSY2D',infile,outfile,paramfile)
    return outfiles

#########################################################################################
def wait_for_files(outfiles):
    import os,glob,time
# pooling on output
    outputdir=os.path.dirname(outfiles[0])
    while (True):
        time.sleep(max(1,WAIT_DELAY))    # wait not less than 1 sec
        print (outputdir+"/*.gs2")
        list= glob.glob(os.path.join(outputdir,"*.gs2")) # get file list
        found = 1
        for i in outputfiles:
            if not (i in list):
                print i,"not found"
                found = 0
                break
        if not found:
            continue
        else:
            break
    
#########################################################################################
def launch(command,input,outfile,paramfile):
    import os.path as op
    outputdir=op.abspath(op.dirname(input))
    call="%s %s %s %s %s %s %s"%(PREBATCH,NPK,command,input,outfile,paramfile,POSTBATCH)
    print call
    # for the moment, do not make the call, simply print out the commands. You can then copy paste to the terminal

#########################################################################################
def add_files( list_of_files, list_of_coefficients=[] ):
    """ add a list of files weighted by the given coefficients
    
    if coefficients are lacking, no weighting is made
    """

    if list_of_coefficients==[]:
        list_of_coefficients=[1.0]*len(list_of_files)
    if len(list_of_files) != len(list_of_coefficients):
        raise "the two lists should be of the same length"
    # check first
    read(list_of_files.pop())
    dloc = get_dim()
    mult(list_of_coefficients.pop())
    put("data")
    # all the others
    for i in range(len(list_of_coefficients)):
        read(list_of_files.pop())
        if (dloc != get_dim()):
            raise "datasets are not of the same dimension"
        mult(list_of_coefficients.pop())
        adddata()
        put("data")
    
#########################################################################################
def main():
# get arguments
    import time
    import sys
    t = time.clock()
    try:
        input=sys.argv[1]
        print "input:",input
        output=sys.argv[2]
        print "output:",output
        param=sys.argv[3]
        print "param-file:",param
    except:
        raise "usage: NPK -e batch_dosy.py file.gs2 dosy.gs2 preset.gtb { N_JOBS=8 }"
    nchunk=N_JOBS
    try:
        argopt=sys.argv[4]
        (key,val)=argopt.split("=")
        if (key=="N_JOBS"):
            nchunk=int(val)
    except:
        pass
    print "njobs:",nchunk
# then do the work
    outfiles = start_batch(input,output,param,nchunk)
    wait_for_files(outfiles)
# found all of them - create a job to add all files together
    add_files( outfiles)
# and exit
    writec(output)
    print "batch_dosy terminated"
    print "Elapsed time : %i seconds"%(time.clock()-t)

#########################################################################################
if __name__ == "__main__":
    main()
