#   CSA fitting Tcl routines
#   Copyright (C) 1999 Mads Bak
#
#   This file is part of the SIMPSON General NMR Simulation Package
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version. 
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Tcl support routines for the C-code in csafit.c
#   

proc showiso {} {
  global g f

  set lst {}
  lappend lst "srate: $g(srate)"
  for {set n 1} {$n <= $f(npeaks)} {incr n} {  
    set iso [expr $g(iso$n)]
    set isoppm [expr $iso / $f(spec-freq)]
    set anisoppm [expr $g(aniso$n) / $f(spec-freq)]
    set lb [expr double(abs($g(gauss$n))) + double(abs($g(lorentz$n)))]
    set frac [expr abs(double($g(gauss$n)) / double($lb))]
    lappend lst [format "   %d) %5.2f ppm    %6.2f ppm    %.3f    %5.1f Hz/%4.2f    %4.1f%%" \
          $n $isoppm $anisoppm $g(eta$n) $lb $frac [expr $g(area$n) / $g(sumarea) * 100.0]]
  }
  return $lst
}


proc evalfunc {val {skip 1}} {
  global f g

  foreach i $val {
    set g([lindex $i 0]) [lindex $i 1]
  }
  csasetspinrate $g(srate)
  set f1 [fcreate -np $f(zerofill) -type spe -sw $f(specwidth) -ref $f(ref-scale)]
  set g(sumarea) 0.0
  for {set n 1} {$n <= $f(npeaks)} {incr n} {
    set intlb {}
    csaspec $g(aniso$n) $g(eta$n)
    set int [csareturnint $g(iso$n) [expr abs($g(scale$n))]]
    set lb [expr double(abs($g(gauss$n))) + double(abs($g(lorentz$n)))]
    set lg [expr abs(double($g(gauss$n)) / double($lb))]
    foreach i $int {
       set ssb [expr abs([lindex $i 0])]
       lappend intlb [concat [lrange $i 1 2 ] $lb $lg]
    }
    set arealist [faddpeaks $f1 $f(truncate) $intlb]
    set area 0.0
    foreach a $arealist {
       set area [expr $area + $a]
    }
    set g(area$n) $area
    set g(sumarea) [expr $g(sumarea) + $g(area$n)]
  }
  if $f(save) {
     puts "Saving $f(savename) ..."
     fsave $f1 $f(savename)     
     set f(save) 0
  }
  set rms [frms $f1 $f(expspe) $f(range) -re]  
  funload $f1

  if [expr ($f(iter) % 10) == 0] {
     puts -nonewline "\nIter: $f(iter)  rms: [format %.7f $rms]  "
     set f(showiso) [showiso]
     set f(rms) $rms
     puts [join $f(showiso) "\n"]
     flush stdout
  }
  return  $rms
}

proc setbestvalues {} {
  global f
  array set ary [split $f(bestval)]
  set lst {}
  foreach i $f(values) {
    set name [lindex $i 0]
    lappend lst [concat $name $ary($name) [lrange $i 2 999]]
  }
  set f(values) $lst
}

proc loadpar {parfile} {
  global f
  puts -nonewline "Loading $parfile"
  set fp [open $parfile.par]
  set f(values) {}
  while {[gets $fp s] != -1} {
    if [llength $s] {
      regsub -all {[ ]+} [string trim $s] { } s
      lappend f(values) $s
    }
  }
  close $fp
  set n 0
  foreach i $f(values) {
    if [string match iso* [lindex $i 0]] {
      incr n
    }
  }
  set f(npeaks) $n
  flush stdout
}

proc setdefpar {} {
  global f par
  if ![info exists f(save)] {set f(save) 0}
  if ![info exists f(name)] {set f(name) fit}
  if ![info exists f(savename)] {set f(savename) $par(name).spe }
  if ![info exists f(skip)] {set f(skip) 1}
  if ![info exists f(function)] {set f(function) fitfunc}
  if ![info exists f(method)] {set f(method) simplex }
  if ![info exists f(max-change)] {set f(max-change) 0.0}
  if ![info exists f(max-change-iter)] {set f(max-change-iter) 30}
  if ![info exists f(max-value)] {set f(max-value) 999999}
  if ![info exists f(max-iter)] {set f(max-iter) 999999}
  if ![info exists f(output-name)] {set f(output-name) peaks}
  if ![info exists f(save-per-iter)] {set f(save-per-iter) 100 }
  if ![info exists f(range)] {set f(range) {{-1e99 1e99}}}
  if ![info exists f(text)] {set f(text) {"Simulation"}}
  if ![info exists f(iter)] {set f(iter) 0}
}

proc savepar {} {
  global f
  
  setdefpar
  puts "Saving $f(name).par and $f(name).spe"
  flush stdout
  set fp [open $f(name).par w]
  foreach i $f(values) {
    foreach j $i {
      puts -nonewline $fp [format "%15s" $j]
    }
    puts $fp "" 
  }
  close $fp
  set lst {}
  foreach i $f(values) {
    lappend lst [lrange $i 0 1]
  }
  set f(save) 1
  set f(savename) $f(name).spe
  evalfunc $lst $f(skip)
} 

proc savepeaks {} {
  global f

  setdefpar
  puts "Saving $f(name).par"
  flush stdout
  set fp [open $f(output-name).par w]
  foreach i $f(values) {
    foreach j $i {
      puts -nonewline $fp [format "%15s" $j]
    }
    puts $fp "" 
  }
  close $fp

  puts "Saving $f(name).spe"
  flush stdout
  set lst {}
  foreach i $f(values) {
    lappend lst [lrange $i 0 1]
  }
  set f(save) 1
  set f(savename) $f(name).spe
  evalfunc $lst $f(skip)

  puts "Saving $f(name).txt"
  flush stdout
  set fp [open $f(name).txt w]
  puts $fp [join $f(text) \n]
  set form [format "%6.4f" $f(rms)]
  set srate [lindex [lindex $f(showiso) 0] 1]
  puts $fp "Simulated spectrum (rms: $form, spinrate: $srate) with chemical shift, anisotropy"
  puts $fp "(defined as dzz-diso), asymmetry, linebroadening/lorentz fraction and intensity: "
  puts $fp [join [lrange $f(showiso) 1 999] \n]
  close $fp

  set npeaks $f(npeaks)

  for {set site 1} {$site <= $npeaks} {incr site} {
    set other {}
    set lst {}

    foreach i $f(values) {   
      set nam [lindex $i 0]
      if [regexp ^\[a-zA-Z_\]*$ $nam] {
        lappend other [lrange $i 0 1]
      } elseif [regexp ^\[a-zA-Z_\]*${site}$ $nam] {
        regsub ^(\[a-zA-Z_\]*)(${site})$ $nam {\1} base
        set base "${base}1"
        lappend lst "$base [lindex $i 1]"
      }
    }
    if [llength $lst] {
       set lst [concat $other $lst]
       set f(npeaks) 1
       puts "Saving $f(name)-$site.spe"
       flush stdout
       set f(save) 1
       set f(savename) $f(name)-$site.spe
       evalfunc $lst $f(skip)
    } else {
      break
    }
  }
  set f(npeaks) $npeaks
} 

proc fitfunc {val} {
  global f stop

  if {$f(iter) > $f(max-iter)} {
    puts "Maxiter reached"
    set stop 1
  }
  if $stop {
     set stop 0
     puts "Printing and saving best fit."
     set f(iter) 0
     setbestvalues
     savepar
     puts ""
     foreach i $f(values) {
        foreach j $i {
          puts -nonewline [format "%15s" $j]
        }
        puts "" 
     }
     puts ""
     puts stderr "Program terminated."
     exit
  }
  incr f(iter)
  set rms [evalfunc $val $f(skip)]
  if [expr $rms < $f(bestrms)] {
    set f(bestrms) $rms
    set f(bestval) $val
  }
  if [expr ($f(iter) % $f(save-per-iter)) == 0] {
    setbestvalues
    savepar
  }
  return $rms
}
 

proc fitpar {} {
  global f

  setdefpar

  set f(bestrms) 1e99
  set f(bestval) {}
  set f(iter) 0
  fit f
  setbestvalues
}

proc tolist {lst} {
  set l {}
  foreach j [split $lst "\n"] {
    if [llength $j] {
      lappend l $j
    }
  }
  return $l
}

proc setfitvar {var val} {
  global f
  set lst {}
  foreach i $f(values) {
     set nam [lindex $i 0]
     if [string match $var $nam] {
       lappend lst [list $nam $val [lindex $i 2] 1]
     } else {
       lappend lst $i
     }
  }
  set f(values) $lst
}

proc release args {
  global f
  foreach var $args {
    set lst {}
    foreach i $f(values) {
       set nam [lindex $i 0]
       if [string match $var $nam] {
         lappend lst [list $nam [lindex $i 1] [lindex $i 2] 1]
       } else {
         lappend lst $i
       }
    }
    set f(values) $lst
  }
}

proc fix args {
  global f
  foreach var $args {
    set lst {}
    foreach i $f(values) {
       set nam [lindex $i 0]
       if [string match $var $nam] {
         lappend lst [list $nam [lindex $i 1] [lindex $i 2] 0]
       } else {
         lappend lst $i
       }
    }
    set f(values) $lst
  }
}

proc setpar {lst} {
  global f

  set n 0
  set f(values) {}
  foreach i [split $lst "\n"] {
    if [llength $i] {
      lappend f(values) $i
      if [string match iso* [lindex $i 0]] {
        incr n
      }
    }
  }
  set f(npeaks) $n
}
