#   Main Tcl procedure and inclusion of statically linked Tcl-code
#   Copyright (C) 1999 Mads Bak
#
#   This file is part of the SIMPSON General NMR Simulation Package
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version. 
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Evaluates statically linked Tcl code.
#   Evaluates the input file.
#   Runs the 'main' program (specified by the user or the default one)

if [catch {
  uplevel #0 {

    # State of the interpreter
    # markstate: remembers the currently defined variables and procedures
    # savestate file: saves the variables and procedures defined since
    #   last  call to markstate
    proc savestate {} {
      uplevel #0 {
      if [info exists clusterstate] {unset clusterstate}
        set clusterstate {}
        lappend clusterstate "global spinsys par cluster"
        if [info exists omitprocs(pulseq)] {unset omitprocs(pulseq)}
        if [info exists omitprocs(main)] {unset omitprocs(main)}
        foreach name [info vars] {
          if [info exists omitvars($name)] continue
          if [array exists $name] {
             lappend clusterstate  [list array set $name \
               [array get $name]]
          } else {
             if [string compare $name clusterstate] {
               lappend clusterstate  [list set $name [set $name]]
             }
          }
        }
        foreach name [info procs] {
          if [info exists omitprocs($name)] continue
          lappend clusterstate  [list proc $name \
            [info args $name] [info body $name]]
        }
        return [join $clusterstate \n]
      }
    }

    proc markvars {} {
      uplevel #0 {
        set omitvars(omitprocs) 1
        set omitvars(omitvars) 1
        foreach name [info vars] {
           set omitvars($name) 1
        }
      }
    }

    proc markprocs {} {
      uplevel #0 {
        set omitprocs(markstate) 1
        set omitprocs(savestate) 1
        foreach name [info procs] {
           set omitprocs($name) 1
        }
      }
    }


# nmrtec path fixup
if ![info exists simpson_path] {
    if [info exists env(SIMPSON_LIB)] {
      set path $env(SIMPSON_LIB)
    } else {
      set path .
    }
} else {
    set path $simpson_path
}

    set stop 0
    proc evalcode {path {varname {}}} {
	source $path
    }
    # all variables in these input files should be remembered
    # whereas the procedures should not be overwritten
    markvars
    evalcode $path/simpson.tcl  simpson
    evalcode $path/csafit.tcl  csafit
    evalcode $path/ftools.tcl ftools
    evalcode $path/misc.tcl misc
    # everything after this should be remembered.
    markprocs

    if [expr $mainargc > 1] {
       incr argc
       set argv [concat $argv0 $argv]  
    }
   if {$argc < 1} {
      puts "Simpson version $simpson_version, Copyright (C)";
      puts "1999-2000 Mads Bak and Jimmy T. Rasmussen"
      puts "2001 Mads Bak and Thomas Vosegaard"
      puts "2002-2005 Thomas Vosegaard"
      puts "Simpson comes with ABSOLUTELY NO WARRANTY, for details";
      puts "read the COPYING file included in this distribution"
      puts "This is free software, and you are welcome to redistribute"
      puts "it under certain conditions according to the GNU General Public License"
      puts "\nPlease specify an inputfile, optionally with other arguments"
      exit
   }
   if ![string compare [lindex $argv 1] "-server"] {
     serversimpson
   } else {
     regsub {(.*)\.in} [lindex $argv 0] {\1} par(name)
     if [file exists $par(name).in] {
        evalcode $par(name).in input
        main
     } else {
       puts stderr "error: unable to open input file '$par(name).in'"     
       exit
     }
   }
  }
} result] {
  puts stderr  "$result"
}
exit
