#!/usr/bin/tclsh
# This script is to be run with the Tcl interpreter

#    Generation of a Postscript figure from the pulse sequence
#    Copyright (C) 1999 Mads Bak
#
#    This file is part of the SIMPSON General NMR Simulation Package
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#    


set header {
/maxx 593 def
/maxy 842 def

90 rotate
0 maxx neg translate
/Frame {
 /fY1 exch def
 /fX1 exch def
 /fY exch def
 /fX exch def
 newpath
 fX fY moveto
 fX1 fY lineto
 fX1 fY1  lineto
 fX fY1 lineto
 closepath
} def

/Cshow { currentpoint stroke moveto
  dup stringwidth pop -2 div vshift rmoveto show } def

/mchan nchan 1 eq { 2 } {nchan} ifelse def
/pheight maxx marg mchan 1 add mul sub mchan div def
/rffac 0.0 rfmax eq { 0 } { pheight rfmax div } ifelse def
/tfac maxy marg dup add sub tmax div def
/x marg def
/TT [  -1   0   0   0   0  ] def
/OFF [  -1   0   0   0   0  ] def
/XX [  -1   x   x   x   x  ] def
/YY [  -1  maxx marg pheight add 1 mul sub maxx marg pheight add 2 mul sub
           maxx marg pheight add 3 mul sub maxx marg pheight add 4 mul sub ] def

/strh fontsize def

/TF {/Helvetica findfont fontsize scalefont setfont } def
/TF2 {/Helvetica findfont fontsize 2 mul scalefont setfont } def
/SF {/Symbol findfont fontsize scalefont setfont } def


% mimimum and maximum allowed width of pulses

/setdtmin {
  dup dtmin le { pop dtmin } if
  dup dtmax gt { pop dtmax } if
} def

/Round {
   dup dup truncate sub 0.0 eq { cvi } {
   100 mul truncate 100 div } ifelse   
   10 string cvs 
} def

/degshow {
 ph 15 string cvs show 0 3 rmoveto (o) show 0 -3 rmoveto
} def

/phshow {
  /ph exch def
  showxy { 
     0 ph eq 360 ph eq or { (X) show } {
       90 ph eq { (Y) show } {
         180 ph eq { (-X) show } {
           270 ph eq { (-Y) show } {
              ph degshow
           } ifelse
         } ifelse
       } ifelse
     } ifelse
   } {     
     ph degshow 
   } ifelse
} def

/Pulse {

 gsave
 /gr   exch def
 /ph   exch def
 /rf   exch def
 /dur  exch def
 /chan exch def

 /tt TT chan get def
 TT chan tt dur setdtmin add put
 XX chan get tt  tfac mul add YY chan get translate

 /X dur setdtmin tfac mul def
 /Y rf rffac mul def

 0 0 X Y Frame gr setgray fill
 0 0 X Y Frame 0 setgray stroke

 rf 0.0 eq { showdelaylen } { true } ifelse
  {
   TF
   X 0.5 mul 3 add strh 2 div Y add translate
   0 0 moveto
   90 rotate
   rf 0.0 ne {
     ph phshow
   } if  
   showdur {    
     (, ) show dur Round show  
     SF ( m) show TF (s) show 
   } if

   showflip rf 0.0 ne and {
     (, ) show 
     dur 1e6 rf div div 360 mul
     TF Round show 
     0 3 rmoveto (o) show
     0 -3 rmoveto ( flip) show
   } if
 } if
 grestore
  /wasreset 0 def
} def

/Acq {

   acqchannels 0 eq { /i0 1 def /i1 nchan def} {
     /i0 acqchannels def /i1 acqchannels def
   } ifelse

   i0 1 i1 {
     gsave 
     /chan exch def

      /tt TT chan get def TT chan tt put
      XX chan get tt  tfac mul add YY chan get translate

     0 0 moveto
     newpath
     0 -4 2 0 360 arc
     closepath
     0 setgray
     fill
     grestore
   } for
} def

/Offset {
  gsave
    /rf   exch def
    /chan exch def

    /off OFF chan get def 
    OFF chan rf put
    
    rf 0 eq { off 0 ne } { true } ifelse    
    {

      /tt TT chan get def
      TT chan tt put
      XX chan get tt  tfac mul add YY chan get translate

      gsave 
      1 -60 translate
      0 0 moveto
      1 setgray
      newpath
      0 0 moveto
      0 45 lineto
      -10 45 lineto
      -10 0 lineto
      closepath
      fill
      0 0 moveto
      0 setgray
      90 rotate
      TF
      rf 100 div truncate 10 div 15 string cvs
      show ( kHz) show
      grestore

      gsave
      0 -6 translate
      newpath
      0 0 moveto
      -3 -6 lineto
      3 -6 lineto 
      closepath fill
      0 -6 moveto
      0 -10 lineto stroke
      grestore

    } if
  grestore
  /wasreset 0 def
} def

/wasreset 0 def

/Reset {
  wasreset 0 eq {
    1 1 nchan {
     /chan exch def
     /tt TT chan get def
     TT chan tt resetlen add put
    } for
  } if
  /wasreset 1 def
} def

0 setgray
2 setlinewidth


% Print the labels
1 1 nchan {
  gsave
  /chan exch def
  /tt TT chan get def
  XX chan get tt  tfac mul add YY chan get translate
  /nam channam chan get def
  -45 pheight 2 div moveto nam TF2 show
  grestore
} for
  

}

proc setvar {varname data {overwrite 0}} {
  global $varname

  set var $varname
  set data [split $data "\n"]
  foreach lst $data {
    if [string match #* [string trimleft $lst]] continue
    set nam [lindex $lst 0]
    if ![string length $nam] continue
    if !$overwrite {
      if [info exists ${var}($nam)] {
        puts stderr "error: '$nam' already exists in array ${var}"
        exit
      }
    }
    set ${var}($nam) [lrange $lst 1 9999]
  }
}

proc pulseq {} {

}

proc spinsys {data} { 
  global dps

  foreach i [split $data "\n"] {
    if ![string compare [lindex $i 0] channels] {
       set dps(chan) [lrange $i 1 999]
    } 
  }
  if ![info exists dps(chan)] {
    puts stderr "error: the 'spinsys' section must contain a 'channels' element"
    exit
  }
  set dps(nchan) [llength $dps(chan)]
}


proc cmp_length {a b} {
  return [expr [string length $a] < [string length $b]]
}

proc cluster {dummy} {}

proc par {data} {
  global par

  set okpar {
    proton_frequency spin_rate sw sw1 np ni method rotor_angle
    gamma_angles fixed_rep real_spec block_diag detect_operator
    crystal_file start_operator name verbose various variable pulse_sequence
    conjugate_fid dipole_check gamma_zero use_cluster cluster_port
    cluster_username
  }
  set allowsubst {
    proton_frequency spin_rate sw sw1 np ni rotor_angle
    gamma_angles fixed_rep real_spec block_diag gamma_zero
    variable
  }
  set data [split $data "\n"]
  foreach lst $data {
    if [string match #* [string trimleft $lst]] continue
    set nam [lindex $lst 0]
    set orignam $nam
    if ![string length $nam] continue
    if {[lsearch -exact $okpar $nam] == -1} {
      puts stderr "error: unknown name '$nam' in parameter section"
      exit
    }
    if ![string compare $nam variable] {
      set nam [lindex $lst 1]
      set lst [lrange $lst 1 9999]    
    }
    if [info exists par($nam)] {
      puts stderr "error: '$nam' already exists in array par"
      exit
    }   
    set ex [lrange $lst 1 9999]
    if {[lsearch -exact $allowsubst $orignam] != -1} {
      set origex $ex
      regsub -all {([A-Za-z_][A-Za-z0-9_]*\(?)} $ex { \1 } ex
      regsub -all {([0-9\.]+) ([Ee])} $ex {\1\2} ex
      set ex "$ex "
      foreach n [lsort -command cmp_length [array names par]] {
        set i 0
        if {[string first " $n " $ex] != -1} {        
          if ![regsub -all " \($n\) " $ex { $par(\1) } ex] {
             puts stderr "error: unable to match $n in $ex"
             exit
          }
          set ex [subst "$ex"]
        }
      }
      if [catch {set ex [expr $ex]} res] {
        puts stderr "error: $res"
        exit  
      }
    }
    set par($nam) $ex
  }
}


proc pulse args {
  global dps
#  print "pulse $args"
  lappend dps(cmd) [concat pulse $args]
}

proc pulseid args {
  global dps
#  print "pulseid $args"
  lappend dps(cmd) [concat pulseid $args]
}

proc delay args {
  global dps
#  print "delay $args"
  lappend dps(cmd) [concat delay $args]
}

proc offset args {
  global dps
#  print "offset $args"
  lappend dps(cmd) [concat offset $args]
}

proc store {n} {
  global dps
#  print "store $n"

  set dps($n) $dps(cmd)
}

proc prop {n {times 1}} {
  global dps indent

#  print "prop $n"
#  incr indent
  FOR {set j 1} {$j <= $times} {incr j} {
    foreach i $dps($n) {
#       print $i
       lappend dps(cmd) $i
    }
  }
#  incr indent -1
}

proc reset {{tt 0}} {
   global t dps
   print "reset"

   set t $tt   
   set dps(cmd) {}
   lappend dps(allcmd) reset
}

proc acq args {
  global dps indent


#  incr indent
  foreach i $dps(cmd) {
     print $i
     lappend dps(allcmd) $i
  }
#  incr indent -1
  lappend dps(allcmd) "acq"
  print "acq $args"
  set dps(cmd) {}
}

proc cmd2ps {ps args} {
  global t dps

  set phases(X) 0
  set phases(Y) 90
  set phases(-X) 180
  set phases(-Y) 270

  set phases(x) 0
  set phases(y) 90
  set phases(-x) 180
  set phases(-y) 270

  set args [eval [concat split $args]]
  set name [lindex $args 0]
  set args [lrange $args 1 999]

  if ![string compare $name pulse] {

    set duration [lindex $args 0]
    for {set i 1} {$i <= $dps(nchan)} {incr i} {
      set rf [lindex $args [expr 2*($i-1)+1]]      
      set ph [lindex $args [expr 2*($i-1)+2]]
      if [info exists phases($ph)] { set ph $phases($ph) }
      puts $ps "$i $duration $rf $ph 0.95 Pulse"
    }
  
  } elseif ![string compare $name pulseid] {

    set duration [lindex $args 0]
    for {set i 1} {$i <= $dps(nchan)} {incr i} {
      set rf [lindex $args [expr 2*($i-1)+1]]
      set ph [lindex $args [expr 2*($i-1)+2]]
      if [info exists phases($ph)] { set ph $phases($ph) }
      puts $ps "$i $duration $rf $ph 0.80 Pulse"
    }

  } elseif ![string compare $name delay] {

    set duration [lindex $args 0]
    for {set i 1} {$i <= $dps(nchan)} {incr i} {
      puts $ps "$i $duration 0 0 0.95 Pulse"
    }
  
  } elseif ![string compare $name offset] {

    for {set i 1} {$i <= $dps(nchan)} {incr i} {
      set off [lindex $args [expr $i-1]]
      puts $ps "$i $off Offset"
    }

  } elseif ![string compare $name reset] {

    puts $ps "Reset"

  } elseif ![string compare $name acq] {

    puts $ps "Acq"

  } else {
    puts stderr "error: unknown command '$name'"
    exit
  }
}

proc getmaxvalues {allargs} {
  global dps

  set dps(tmax) 0
  set dps(rfmax) 0

  set last {}
  foreach args $allargs {    
    set name [lindex $args 0]
    set args [lrange $args 1 999]

    if ![string compare $last reset] {
      set last $name
      continue
    }
    set last $name

    if ![string compare $name pulse]  {

      set duration [lindex $args 0]
      for {set i 1} {$i <= $dps(nchan)} {incr i} {
        set rf [lindex $args [expr 2*($i-1)+1]]
        if [expr $rf > $dps(rfmax)] {set dps(rfmax) $rf}
      }
      if [expr $duration < $dps(dtmin)] {set duration $dps(dtmin)}
      if [expr $duration > $dps(dtmax)] {set duration $dps(dtmax)}
      set dps(tmax) [expr $dps(tmax) + $duration]

    } elseif ![string compare $name pulseid] {

      set duration [lindex $args 0]
      for {set i 1} {$i <= $dps(nchan)} {incr i} {
        set rf [lindex $args [expr 2*($i-1)+1]]
        if [expr $rf > $dps(rfmax)] {set dps(rfmax) $rf}
      }
      if [expr $duration < $dps(dtmin)] {set duration $dps(dtmin)}
      if [expr $duration > $dps(dtmax)] {set duration $dps(dtmax)}
      set dps(tmax) [expr $dps(tmax) + $duration]

    } elseif ![string compare $name delay] {
      set duration [lindex $args 0]
      if [expr $duration < $dps(dtmin)] {set duration $dps(dtmin)}
      if [expr $duration > $dps(dtmax)] {set duration $dps(dtmax)}
      set dps(tmax) [expr $dps(tmax) + $duration]
    } elseif ![string compare $name offset] {
    } elseif ![string compare $name acq] {
    } elseif ![string compare $name reset] {
      set dps(tmax) [expr $dps(tmax) + $dps(resetlen)]
    } else {
      puts stderr "error: unknown command '$name'"
      exit
    }
  }
}


proc simpson args {
  global par
  reset
  if ![info exists par(pulse-sequence)] {
    set par(pulse-sequence) pulseq
  }
  uplevel $par(pulse-sequence)
}

proc fsimpson args {
  simpson
  return 0
}

proc print {args} {
   global indent
#   puts -nonewline ""
   FOR {set i 1} {$i <= $indent} {incr i} {
     puts -nonewline "    "
   }
   eval [concat puts $args]
}

set quiet 0

set indent 0

set dps(dtmax) 100
set dps(dtmin) 5
set dps(resetlen) 20
set dps(marg) 100
set dps(fontsize) 10

set dps(showdur) true
set dps(showdelaylen) true
set dps(showflip) true
set dps(showxy) true


set par(name) {}

# if called from inside the simdps.c file
if [info exists mainargc] {
  if [expr $mainargc > 1] {
     incr argc
     set argv [concat $argv0 $argv]  
  }
}

for {set i 0} {$i < $argc} {incr i} {
  set arg [lindex $argv $i]

  if ![string compare $arg {-maxlen}] {
    incr i
    set arg [lindex $argv $i]
    if ![string length $arg] {
      puts stderr "error: a number must follow -maxlen"
      exit
    }
    set dps(dtmax) $arg
  } elseif ![string compare [lindex $argv $i] {-minlen}] {
    incr i
    set arg [lindex $argv $i]
    if ![string length $arg] {
      puts stderr "error: a number must follow -minlen"
      exit
    }
    set dps(dtmin) $arg
  } elseif ![string compare [lindex $argv $i] {-resetlen}] {
    incr i
    set arg [lindex $argv $i]
    if ![string length $arg] {
      puts stderr "error: a number must follow -resetlen"
      exit
    }
    set dps(resetlen) $arg
  } elseif ![string compare [lindex $argv $i] {-fontsize}] {
    incr i
    set arg [lindex $argv $i]
    if ![string length $arg] {
      puts stderr "error: a number must follow -fontsize"
      exit
    }
    set dps(fontsize) $arg
  } elseif ![string compare [lindex $argv $i] {-marg}] {
    incr i
    set arg [lindex $argv $i]
    if ![string length $arg] {
      puts stderr "error: a number must follow -marg"
      exit
    }
    set dps(marg) $arg
  } elseif ![string compare [lindex $argv $i] {-nodur}] {
    set dps(showdur) false
  } elseif ![string compare [lindex $argv $i] {-noflip}] {
    set dps(showflip) false
  } elseif ![string compare [lindex $argv $i] {-noxy}] {
    set dps(showxy) false
  } elseif ![string compare [lindex $argv $i] {-nodelaylen}] {
    set dps(showdelaylen) false
  } else {
    set par(name) [lindex $argv $i]
  }
}

if ![string length $par(name)] {

  if [info exists simpson_version] {
    set version "- Version $simpson_version"
  } else {
    set version "- Script version"
  }

puts "
  Copyright (c) 1999 Mads Bak $version

  simdps: Displays a SIMPSON pulse sequence. Generates a Postscript
       plot showing a graphical view of the pulse sequence as
       rows of shaded bars corresponding to the rf-field strengths
       and length on the affected spectrometer channels.

  Usage
       simdps file.in \[options\]

  Options:

    -minlen v   : Change the minimum width of the displayed pulses in usec.
                  $dps(dtmin) is the default value.
    -maxlen v   : Change the maximum width of the displayed pulses in usec.
                  $dps(dtmax) is the default value.
    -resetlen v : Change additional space after a 'reset' command, i.e. 
                  restart of the pulse sequence.
                  $dps(resetlen) is the default value.
    -fontsize v : Change font size of text.
                  $dps(fontsize) is the default value.
    -marg v     : Change margin around plot
                  $dps(marg) is the default value.
    -noflip     : Do not display flip angles
    -nodur      : Do not display duration values.
    -nodelaylen : Do not display labels at delays.
    -noxy       : Do not display phases as X,Y, -X or -Y

   Note:
     Remember to reduce the number of sampling points before running
     this program.
"
  exit
} 
regsub {(.*)\.in} $par(name) {\1} par(name)   

if ![file exists $par(name).in] {
  puts "dps error: unable to open file '$par(name).in'"
  exit
}

set dps(allcmd) {}

source $par(name).in

proc unknown args {
  print "unknown $args"
}

rename for FOR

proc for {start test inc body} {
  global dps indent

#  incr indent
#  print "for \{$start\} \{$test\} \{$inc\} \{"
#  incr indent
  uplevel  [list FOR $start $test $inc $body]
#  incr indent -1
#  print "\}"
#  incr indent -1
}

set t 0

main


rename for {}
rename FOR for

if [catch {set ps [open $par(name).ps w]} res] {
  puts stderr "error: unable to create $par(name).ps"
  exit
}

puts $ps "%!PS-Adobe-2.0\n"


# what channel will be shown dots as acquisition on (0 = all)
set dps(acqchannels) 0
if [info exists par(detect-operator)] {
  if [regsub {.*I([0-9n]*).*} $par(detect-operator) {\1} chan] {
    set dps(acqchannels) $chan
    if ![string compare $chan n] {
      set dps(acqchannels) 0 
    }     
  }
}

getmaxvalues $dps(allcmd)

puts $ps "/acqchannels $dps(acqchannels) def"
puts $ps "/tmax $dps(tmax) def"
puts $ps "/nchan $dps(nchan) def"
puts $ps "/rfmax $dps(rfmax) def"
puts $ps "/dtmin $dps(dtmin) def"
puts $ps "/dtmax $dps(dtmax) def"
puts $ps "/resetlen $dps(resetlen) def"
puts $ps "/showdur $dps(showdur) def"
puts $ps "/showdelaylen $dps(showdelaylen) def"
puts $ps "/showflip $dps(showflip) def"
puts $ps "/showxy $dps(showxy) def"
puts $ps "/marg $dps(marg) def"
puts $ps "/fontsize $dps(fontsize) def"

set nam ""
foreach i $dps(chan) {
  set nam "$nam ($i)"
}
puts $ps "/channam \[ -1 $nam \] def"

puts $ps $header

foreach i $dps(allcmd) {
  cmd2ps $ps $i
}
puts $ps "showpage"
close $ps

exit











